@@ -30,6 +30,7 @@
 #include <QFont>
 #include <QResizeEvent>
 #include <QPalette>
+#include <QTimer>
 
 #include <kdebug.h>
 #include <kdialog.h>
@@ -44,7 +45,6 @@
 #include <kiconloader.h>
 
 #include <kabc/addressee.h>
-#include <akonadi/contact/contactsearchjob.h>
 
 // application specific includes
 #include "kraftdb.h"
@@ -77,21 +77,23 @@
 #include "doctype.h"
 #include "catalogtemplate.h"
 #include "importitemdialog.h"
-
-#include <qtimer.h>
+#include "addressprovider.h"
 #include "doclocaledialog.h"
 
 
 KraftViewScroll::KraftViewScroll( QWidget *parent ):
 QScrollArea( parent )
 {
-  widget = new QWidget;
-  setWidget(widget);
-  widget->setAutoFillBackground(false);
+  myWidget = new QWidget;
+  myWidget->setAutoFillBackground(false);
   layout = new QVBoxLayout;
   layout->setAlignment(Qt::AlignTop);
-  widget->setLayout(layout);
+  layout->setSizeConstraint( QLayout::QLayout::SetMinAndMaxSize );
+  layout->setMargin( 0 );
+  myWidget->setLayout(layout);
+  setWidget(myWidget);
   setWidgetResizable(true);
+
 }
 
 void KraftViewScroll::addChild( QWidget *child, int index )
@@ -138,6 +140,11 @@ KraftView::KraftView(QWidget *parent) :
   mDetailHeader->setLineWidth( 1 );
   mDetailHeader->setAutoFillBackground(true);
 
+  mAddressProvider = new AddressProvider( this );
+  connect( mAddressProvider, SIGNAL(addresseeFound( const QString&, const KABC::Addressee& )),
+           this, SLOT( slotAddresseeFound( const QString&, const KABC::Addressee& )));
+
+
   QPalette palette;
   palette.setColor(mDetailHeader->backgroundRole(), QColor( "darkBlue" ));
   palette.setColor(mDetailHeader->foregroundRole(), QColor( "white "));
@@ -162,8 +169,8 @@ KraftView::KraftView(QWidget *parent) :
   mCSplit->addWidget( mAssistant );
 
   /* catalog template selection signal */
-  connect( mAssistant,  SIGNAL( positionSelected( Katalog*, void* ) ),
-           this,  SLOT( slotAddPosition( Katalog*, void* ) ) );
+  connect( mAssistant,  SIGNAL( templatesToDocument(Katalog*,CatalogTemplateList) ),
+           this,  SLOT( slotAddItems( Katalog*, CatalogTemplateList ) ) );
 
   /* signal to toggle the visibility of the template section in the assistant */
   connect(  mAssistant, SIGNAL( toggleShowTemplates( bool ) ),
@@ -194,7 +201,6 @@ KraftView::KraftView(QWidget *parent) :
 
   setupMappers();
 
-  // mNewTemplates.setAutoDelete( true );
 }
 
 KraftView::~KraftView()
@@ -303,7 +309,7 @@ void KraftView::setupPositions()
 
     m_positionScroll = edit->positionScroll();
 
-    connect( edit, SIGNAL( addPositionClicked() ), SLOT( slotAddPosition() ) );
+    connect( edit, SIGNAL( addPositionClicked() ), SLOT( slotAddNewItem() ) );
     connect( edit, SIGNAL( addExtraClicked() ), SLOT( slotAddExtraPosition() ) );
     connect( edit, SIGNAL( importItemsClicked() ), SLOT( slotImportItems() ) );
 
@@ -329,9 +335,8 @@ void KraftView::redrawDocument( )
 
     kDebug() << "Loaded address uid from database " << mContactUid << endl;
     if( ! mContactUid.isEmpty() ) {
-      // FIXME - use centralised address provider
-      Akonadi::ContactSearchJob *job = new Akonadi::ContactSearchJob;
-      connect( job, SIGNAL( result( KJob* ) ), SLOT( readContacts( KJob* ) ) );
+
+      mAddressProvider->getAddressee( mContactUid );
     }
 
     if( !address.isEmpty() ) {
@@ -362,25 +367,11 @@ void KraftView::redrawDocument( )
     mModified = false;
 }
 
-void KraftView::readContacts( KJob *job )
+void KraftView::slotAddresseeFound( const QString& uid, const KABC::Addressee& contact )
 {
-  kDebug() << "Reading Akonadi Search Job!";
-  if ( job->error() ) {
-    qDebug() << "Akonadi Contact Job Read Error: " << job->errorString();
-    return;
-  }
-
-  Akonadi::ContactSearchJob *searchJob = qobject_cast<Akonadi::ContactSearchJob*>( job );
-
-  // iterate over all found contacts and and find the one with the correct uid
-  foreach ( const KABC::Addressee &contact, searchJob->contacts() ) {
-    const QString uid = contact.uid();
-    if( uid == mContactUid ) {
-      slotNewAddress( contact );
-      kDebug() << "The loaded Contact has this realname: " << contact.realName() << endl;
-      break;
-    }
-  }
+  kDebug() << "Addressee Found with uid " << uid;
+  slotNewAddress( contact, false );
+  kDebug() << "The loaded Contact has this realname: " << contact.realName() << endl;
 }
 
 void KraftView::redrawDocPositions( )
@@ -763,39 +754,43 @@ void KraftView::slotPositionModified( int pos )
   QTimer::singleShot( 0, this, SLOT( refreshPostCard() ) );
 }
 
-void KraftView::slotNewAddress( const Addressee& contact )
+void KraftView::slotNewAddress( const Addressee& contact, bool interactive )
 {
+
   Addressee adr( contact );
 
   if( contact.isEmpty() ) {
     return;
   }
+  QString newAddress = mAddressProvider->formattedAddress( contact );
+  const QString currAddress = m_headerEdit->m_postAddressEdit->toPlainText();
 
-  if( ! adr.isEmpty() ) {
-    m_headerEdit->m_labName->setText( adr.realName() );
+  bool replace = true;
 
-    Address address;
-
-    Address::List addresses = adr.addresses();
-    if ( addresses.count() > 1 ) {
-      kDebug() << "Have more than one address, taking the default add" << endl;
-      address = adr.address( KABC::Address::Pref );
-    } else if ( addresses.count() == 0 ) {
-      kDebug() << "Have no address, problem!" << endl;
+  if( currAddress.isEmpty() ) {
+    replace = true;
+  } else if( currAddress != newAddress ) {
+    // non empty and current different from new address
+    // need to ask first if we overwrite
+    if( interactive ) {
+      if( KMessageBox::questionYesNo( this, i18n("The address label is not empty. Do you really want to replace it?"),
+                                     i18n("Address Overwrite") ) == KMessageBox::No ) replace = false;
     } else {
-      // FIXME: Handle multiple addresses
-      address = addresses.first();
+      // this happens when the document is loaded and the address arrives from addressbook
+      replace = false;
     }
+  } else if( currAddress == newAddress ) {
+    // both are equal, no action needed
+    return;
+  }
 
+  if( replace ) {
+    m_headerEdit->m_labName->setText( contact.realName() );
     mContactUid = contact.uid();
-    QString adrStr;
-    if( address.label().isEmpty() ) {
-      adrStr = address.formattedAddress( adr.realName() );
-    } else {
-      adrStr = address.label();
-    }
-    kDebug() << "formatted address string: " << adrStr << endl;
-    m_headerEdit->m_postAddressEdit->setText( adrStr );
+
+    m_headerEdit->m_postAddressEdit->setText( newAddress );
+
+    // Generate the welcome
     m_headerEdit->m_letterHead->clear();
     QStringList li = generateLetterHead( adr );
 
@@ -861,14 +856,23 @@ void KraftView::slotNewFooterText( const QString& str )
   slotModifiedFooter();
 }
 
-void KraftView::slotAddPosition()
+// Add a new item. A katalog is required if user wants to store it in a
+// catalog immediately. FIXME - now the current active catalog in the
+// catalog selection is used.
+void KraftView::slotAddNewItem()
 {
-  // find the katalog
-  CatalogSelection *catsel = mAssistant->catalogSelection();
-  slotAddPosition( catsel->currentSelectedKat(), 0 );
+  Katalog* kat = mAssistant->catalogSelection()->currentSelectedKat();
+  slotAddItem( kat, 0 );
 }
 
-void KraftView::slotAddPosition( Katalog *kat, void *tmpl )
+void KraftView::slotAddItems( Katalog *kat, CatalogTemplateList templates)
+{
+  foreach( CatalogTemplate *templ, templates ) {
+    slotAddItem( kat, templ );
+  }
+}
+
+void KraftView::slotAddItem( Katalog *kat, CatalogTemplate *tmpl )
 {
   // newpos is a list position, starts counting at zero!
   int newpos = mPositionWidgetList.count();
@@ -897,7 +901,7 @@ void KraftView::slotAddPosition( Katalog *kat, void *tmpl )
         dia = new InsertTemplDialog( this );
         FloskelTemplate *ftmpl = static_cast<FloskelTemplate*>( tmpl );
         dp->setText( ftmpl->getText() );
-        dp->setUnit( ftmpl->einheit() );
+        dp->setUnit( ftmpl->unit() );
         dp->setUnitPrice( ftmpl->unitPrice() );
 
         s = KraftSettings::self()->templateToPosDialogSize();
@@ -947,18 +951,18 @@ void KraftView::slotAddPosition( Katalog *kat, void *tmpl )
 
         // if it's a new position, create a catalog template in the incoming chapter
         if ( newTemplate ) {
-          const QString chapter = dia->chapter();
-          if ( !chapter.isEmpty() ) {
-            FloskelTemplate *flos = new FloskelTemplate( -1, dp->text(),
-                                                         dp->unit().id(),
-                                                         kat->chapterID( chapter ),
-                                                         1, /* CalcKind = Manual */
-                                                         QDateTime::currentDateTime(),
-                                                         QDateTime::currentDateTime() );
-            flos->setManualPrice( dp->unitPrice() );
-            flos->save(); // <- Checke das hier!
-            mNewTemplates.append( flos );
-          }
+          // const QString chapter = dia->chapter();
+          int chapterId = 0; // FIXME - need a new way of chapter picking
+
+          FloskelTemplate *flos = new FloskelTemplate( -1, dp->text(),
+                                                       dp->unit().id(),
+                                                       chapterId,
+                                                       1 /* CalcKind = Manual */ );
+
+          flos->setManualPrice( dp->unitPrice() );
+          flos->save(); // <- Checke das hier!
+          mNewTemplates.append( flos );
+
         }
       } else if ( kat->type() == MaterialCatalog ) {
         KraftSettings::self()->setTemplateToPosDialogSize( s );
@@ -1350,10 +1354,12 @@ void KraftView::saveChanges()
 
 void KraftView::slotFocusPosition( PositionViewWidget *posWidget, int pos )
 {
-  kDebug() << "Focussing on widget " << posWidget << " on pos " << pos << endl;
   if( posWidget && pos > 0) {
     int w = posWidget->height();
-    m_positionScroll->ensureVisible( 0, (pos)*w, 0, w );
+    kDebug() << "Focussing on widget " << posWidget << " on pos " << pos << " with heigt " << w;
+    int y = (pos)*w;
+    kDebug() << "Focus on y-koord " << y << " on canvas height " << (m_positionScroll->widget())->height();
+    m_positionScroll->ensureVisible( 2, y, 0, 0 );
   } else {
     m_positionScroll->ensureVisible( 0, 0 );
   }
@@ -1399,4 +1405,3 @@ void KraftView::print(QPrinter * /* pPrinter */ )
 
 }
 
-#include "kraftview.moc"
