@@ -73,6 +73,7 @@
 #include "kraftview_ro.h"
 #include "databasesettings.h"
 #include "setupassistant.h"
+#include "addressprovider.h"
 
 #define ID_STATUS_MSG 1
 
@@ -94,6 +95,10 @@ Portal::Portal( QWidget *parent, KCmdLineArgs *args, const char* name)
   editCopy->setEnabled(false);
   editPaste->setEnabled(false);
 
+  mAddressProvider = new AddressProvider( this );
+  connect( mAddressProvider, SIGNAL( addresseeFound( const QString&, const KABC::Addressee&)),
+          this, SLOT( slotReceivedMyAddress( const QString&, const KABC::Addressee& ) ) );
+
   setAutoSaveSettings();
   QTimer::singleShot( 0, this, SLOT( slotStartupChecks() ) );
 }
@@ -261,13 +266,15 @@ void Portal::slotStartupChecks()
   if ( !Akonadi::Control::start( this ) ) {
     kError() << "Failed to start Akonadi!";
   }
-
+  Akonadi::Control::widgetNeedsAkonadi( this );
+  // FIXME: This starts Akonadi and such, check if the decision if the setup
+  // assistant is really needed can't be pulled here.
   SetupAssistant assi(this);
   if( assi.init( SetupAssistant::Update) ) {
     assi.exec();
   }
 
- if( ! KraftDB::self()->isOk() ) {
+  if( ! KraftDB::self()->isOk() ) {
     QSqlError err = KraftDB::self()->lastError();
     kDebug() << "The last sql error id: " << err.type() << endl;
 
@@ -321,10 +328,32 @@ void Portal::slotStartupChecks()
       mCmdLineArgs->clear();
     }
 
+    // Fetch my address
+    QString myUid = KraftSettings::self()->userUid();
+    if( myUid.isEmpty() ) {
+      QString myName = KraftSettings::self()->userName();
+      kDebug() << "Got My Name: " << myName;
+      mAddressProvider->getAddresseeByName( myName );
+    } else {
+      kDebug() << "Got My UID: " << myUid;
+      mAddressProvider->getAddressee( myUid );
+    }
+
     slotStatusMsg( i18n( "Ready." ) );
   }
 }
 
+void Portal::slotReceivedMyAddress( const QString& uid, const KABC::Addressee& contact )
+{
+  myContact = contact;
+
+  KraftSettings::self()->setUserUid( contact.uid() );
+  KraftSettings::self()->writeConfig();
+
+  kDebug() << "Received my address: " << contact.realName() << "(" << uid << ")";
+  ReportGenerator::self()->setMyContact( contact );
+}
+
 bool Portal::queryClose()
 {
   return true;
@@ -358,11 +387,11 @@ void Portal::slotNewDocument()
   wiz.init();
   if ( wiz.exec() ) {
     DocumentMan *docman = DocumentMan::self();
-    DocGuardedPtr doc = docman->createDocument();
+    DocGuardedPtr doc = docman->createDocument( wiz.docType() );
 
     doc->setDate( wiz.date() );
     doc->setAddressUid( wiz.addressUid() );
-    doc->setDocType( wiz.docType() );
+    // doc->setDocType( wiz.docType() );
     doc->setWhiteboard( wiz.whiteboard() );
     createView( doc );
   }
@@ -419,7 +448,7 @@ void Portal::slotCopyDocument( const QString& id )
     doc->setWhiteboard( wiz.whiteboard() );
     doc->setAddressUid( wiz.addressUid() );
     doc->saveDocument();
-    m_portalView->slotDocumentCreated( doc );
+    m_portalView->docDigestView()->slotUpdateView();
     kDebug() << "Document created from id " << id << ", saved with id " << doc->docID().toString() << endl;
   }
 }
@@ -485,7 +514,7 @@ void Portal::slotPrintDocument()
     ArchiveMan *archman = ArchiveMan::self();
     dbID archID = archman->archiveDocument( docPtr );
     slotPrintDocument( ident, archID );
-    m_portalView->docDigestView()->addArchivedItem(docPtr->docID(), archID);
+    // m_portalView->docDigestView()->addArchivedItem(docPtr->docID(), archID);
   }
   busyCursor( false );
   slotStatusMsg( i18n( "Ready." ) );
@@ -545,11 +574,11 @@ void Portal::slotPrintDocument( const QString& id,  const dbID& archID )
 {
   if ( archID.isOk() ) {
     slotStatusMsg(i18n("Printing archived document...") );
-    mReportGenerator = ReportGenerator::self();
-    connect( mReportGenerator, SIGNAL( pdfAvailable( const QString& ) ),
+    ReportGenerator *repGen = ReportGenerator::self();
+    connect( repGen, SIGNAL( pdfAvailable( const QString& ) ),
              this,  SLOT( slotOpenPdf( const QString& ) ) );
 
-    mReportGenerator->createPdfFromArchive( id, archID ); // work on document identifier.
+    repGen->createPdfFromArchive( id, archID ); // work on document identifier.
   }
 }
 
@@ -562,8 +591,8 @@ void Portal::slotOpenPdf( const QString& fileName )
 
 void Portal::slotOpenDocument( const QString& id )
 {
-  slotStatusMsg(i18n("Opening document..."));
-
+  slotStatusMsg( i18n("Opening document %1").arg(id ) );
+  kDebug() << "Opening document " << id;
   if( !id.isEmpty() ) {
     DocumentMan *docman = DocumentMan::self();
     DocGuardedPtr doc = docman->openDocument( id );
@@ -599,9 +628,9 @@ void Portal::slotDocumentSelected( const QString& doc )
 void Portal::slotArchivedDocExecuted()
 {
 
-  ArchDocDigest dig = m_portalView->docDigestView()->currentArchiveDoc();
+  // ArchDocDigest dig = m_portalView->docDigestView()->currentArchiveDoc();
 
-  slotOpenArchivedDoc( dig );
+  // slotOpenArchivedDoc( dig );
 }
 
 void Portal::slotArchivedDocSelected( const ArchDocDigest& )
@@ -687,12 +716,11 @@ void Portal::slotViewClosed( bool success, DocGuardedPtr doc )
 {
   // doc is only valid on success!
   if ( doc && success )  {
+    DocDigestView *dv = m_portalView->docDigestView();
+    kDebug() << "DocDigestView: " << dv;
+
+    dv->slotUpdateView();
     kDebug() << "A view was closed saving and doc is new: " << doc->isNew() << endl;
-    if ( doc->isNew() ) {
-      m_portalView->slotDocumentCreated( doc );
-    } else {
-      m_portalView->slotDocumentUpdate( doc );
-    }
   } else {
     kDebug() << "A view was closed canceled" << endl;
   }
