@@ -26,7 +26,6 @@
 #include <kmenu.h>
 #include <kiconloader.h>
 
-#include "docposition.h"
 #include "positionviewwidget.h"
 #include "unitmanager.h"
 #include "geld.h"
@@ -94,9 +93,9 @@ PositionViewWidget::PositionViewWidget()
   // state submenu:
   mStateSubmenu = mExecPopup->addMenu(i18n( "Position Kind" ));
   mStateSubmenu->addAction( i18n( "Normal" ), this, SIGNAL( positionStateNormal() ) );
-  mStateSubmenu->addAction( KIcon( "alternative" ),
+  mStateSubmenu->addAction( KIcon( "kraft_alternative" ),
                             i18n( "Alternative" ), this, SIGNAL( positionStateAlternative() ) );
-  mStateSubmenu->addAction( KIcon( "demand" ),
+  mStateSubmenu->addAction( KIcon( "kraft_demand" ),
                             i18n( "On Demand" ), this, SIGNAL( positionStateDemand() ) );
 
   mExecPopup->addSeparator();
@@ -130,6 +129,7 @@ PositionViewWidget::PositionViewWidget()
 
   this->setAutoFillBackground(true);
   this->setBaseSize(this->width(), 100);
+  this->layout()->setMargin( 6 );
 }
 
 void PositionViewWidget::setDocPosition( DocPositionBase *dp, KLocale* loc )
@@ -536,7 +536,7 @@ void PositionViewWidget::slotSetPositionAlternative()
   lKind->show();
   lKind->setToolTip( i18n( "This is an alternative position."
                               " Use the position toolbox to change." ) );
-  lKind->setPixmap( SmallIcon( "alternative" ) );
+  lKind->setPixmap( SmallIcon( "kraft_alternative" ) );
   mKind = Alternative;
   slotRefreshPrice();
 
@@ -552,7 +552,7 @@ void PositionViewWidget::slotSetPositionDemand()
   lKind->show();
   lKind->setToolTip( i18n( "This is a as required position. "
                               "Use the position toolbox to change." ) );
-  lKind->setPixmap( SmallIcon( "demand" ) );
+  lKind->setPixmap( SmallIcon( "kraft_demand" ) );
   mKind = Demand;
   slotRefreshPrice();
 
@@ -631,5 +631,3 @@ void PositionViewWidget::paintEvent ( QPaintEvent* )
   // check again: Ui_positionWidget::paintEvent( pe );
 }
 
-#include "positionviewwidget.moc"
-
