@@ -23,6 +23,7 @@
 #include <QList>
 #include <QTextDocument>
 #include <QApplication>
+#include <QTextCodec>
 
 #include <kdebug.h>
 #include <kstandarddirs.h>
@@ -45,7 +46,7 @@
 #include "texttemplate.h"
 #include "defaultprovider.h"
 #include "doctype.h"
-
+#include "addressprovider.h"
 
 ReportGenerator *ReportGenerator::self()
 {
@@ -64,6 +65,12 @@ ReportGenerator::ReportGenerator()
   connect( &mProcess, SIGNAL( readyReadStandardOutput()), this, SLOT( slotReceivedStdout() ) );
   connect( &mProcess, SIGNAL( readyReadStandardError()), this, SLOT( slotReceivedStderr() ) );
   connect( &mProcess, SIGNAL( error ( QProcess::ProcessError )), this, SLOT( slotError( QProcess::ProcessError)));
+
+  mAddressProvider = new AddressProvider( this );
+  connect( mAddressProvider, SIGNAL( addresseeFound( const QString&, const KABC::Addressee& )),
+           this, SLOT( slotAddresseeFound( const QString&, const KABC::Addressee& ) ) );
+  connect( mAddressProvider, SIGNAL( finished(int) ),
+           this, SLOT( slotAddresseeSearchFinished(int)) );
 }
 
 ReportGenerator::~ReportGenerator()
@@ -96,7 +103,13 @@ void ReportGenerator::slotConvertTemplate( const QString& templ )
 
     if ( temp.open() ) {
       QTextStream s(&temp);
-      // s.setCodec( QTextCodec::codecForLocale() );
+
+      // The following explicit coding settings were needed for Qt 4.7.3, former Qt versions
+      // seemed to default on UTF-8. Try to comment the following two lines for older Qt versions
+      // if needed and see if the trml file on the disk still is UTF-8 encoded.
+      QTextCodec *codec = QTextCodec::codecForName("UTF-8");
+      s.setCodec( codec );
+
       s << templ;
     } else {
       kDebug() << "ERROR: Could not open temporar file";
@@ -117,7 +130,7 @@ QString ReportGenerator::findTemplate( const QString& type )
 {
   DocType dType( type );
 
-  QString tmplFile = dType.templateFile();
+  QString tmplFile = dType.templateFile( mArchDoc->locale()->country() );
 
   if ( tmplFile.isEmpty() ) {
     KMessageBox::error( 0, i18n("A document template named %1 could not be loaded. "
@@ -138,49 +151,29 @@ void ReportGenerator::fillupTemplateFromArchive( const dbID& id )
 {
   mArchDoc = new ArchDoc(id);
 
-  // FIXME: Read all contacts, even if the customer is not in the address book
-  // we need to read all for our own address.
-  Akonadi::ContactSearchJob *job = new Akonadi::ContactSearchJob;
-  connect( job, SIGNAL( result( KJob* ) ), SLOT( addressReceived( KJob* ) ) );
+  const QString clientUid = mArchDoc->clientUid();
+  if( ! clientUid.isEmpty() ) {
+    mAddressProvider->getAddressee( clientUid );
+  } else {
+    // no address UID specified, skip the addressee search and generate the template directly
+    slotAddresseeSearchFinished(0);
+  }
 }
 
-// Result Slot of the Contact Search above
-void ReportGenerator::addressReceived( KJob* job )
+void ReportGenerator::slotAddresseeFound( const QString&, const KABC::Addressee& contact )
 {
-  kDebug() << "Reading Akonadi Search Job!";
-  KABC::Addressee addressee ;
-  KABC::Addressee myAddress;
+  mCustomerContact = contact;
+}
 
-  if( ! job ) {
-    kError() << "Akonadi Address search job failed!";
-    return;
-  }
-
-  if ( job->error() ) {
-    qDebug() << "Akonadi Contact Job Read Error: " << job->errorString();
-    return;
-  }
-
-  QString myName = KraftSettings::self()->userName();
-  kDebug() << "MY name is " << myName;
-
-  Akonadi::ContactSearchJob *searchJob = qobject_cast<Akonadi::ContactSearchJob*>( job );
-  KABC::Addressee::List contacts = searchJob->contacts();
-  kDebug() << "Amount of address entries: " << contacts.size();
-
-  // iterate over all found contacts and search for the customer address
-  // and 'My' address
-  foreach ( const KABC::Addressee &contact, contacts ) {
-    if( contact.uid() == mArchDoc->clientUid() ) {
-      addressee = contact;
-    }
-    kDebug() << "XXXXXXXXXX comparing " << contact.name() << " with " << myName;
-    if( contact.name() == myName ) {
-      myAddress = contact;
-    }
-    if( !( myAddress.isEmpty() || addressee.isEmpty() ) ) break;
-  }
+void ReportGenerator::setMyContact( const KABC::Addressee& contact )
+{
+  myContact = contact;
+}
 
+void ReportGenerator::slotAddresseeSearchFinished( int )
+{
+  // now the addressee search through the address provider is finished.
+  // Rendering can be started.
   QString tmplFile = findTemplate( mArchDoc->docType() );
 
   if ( tmplFile.isEmpty() ) {
@@ -197,7 +190,6 @@ void ReportGenerator::addressReceived( KJob* job )
   /* A placeholder has the format <!-- %VALUE --> */
 
   ArchDocPositionList posList = mArchDoc->positions();
-  QString loopResult;
   QString h;
 
   ArchDocPositionList::iterator it;
@@ -211,7 +203,19 @@ void ReportGenerator::addressReceived( KJob* job )
     tmpl.setValue( "POSITIONS", "POS_TEXT",
                    rmlString( pos.text(), QString( "%1text" ).arg( pos.kind().toLower() ) ) );
 
-    h.setNum( pos.amount(), 'f', 2 );
+    // format the amount value of the item, do not show the precision if there is no fraction
+    double amount = pos.amount();
+    QString num;
+    num.setNum( amount ); // no locale awareness.
+    int prec = 0;
+    if( num.contains( QChar('.') ) ) {
+      // there is a decimal point
+      // calculate the precision
+      prec = num.length() - (1+num.lastIndexOf( QChar('.') ) );
+    }
+    // kDebug() << "**** " << num << " has precision " << prec;
+    h = mArchDoc->locale()->formatNumber( amount, prec );
+
     tmpl.setValue( "POSITIONS", "POS_AMOUNT", h );
     tmpl.setValue( "POSITIONS", "POS_UNIT", pos.unit() );
     tmpl.setValue( "POSITIONS", "POS_UNITPRICE", pos.unitPrice().toString( mArchDoc->locale() ) );
@@ -233,33 +237,8 @@ void ReportGenerator::addressReceived( KJob* job )
   tmpl.setValue( TAG( "DOCTYPE" ), mArchDoc->docType() );
   tmpl.setValue( TAG( "ADDRESS" ), mArchDoc->address() );
 
-  tmpl.setValue( TAG( "CLIENT_NAME" ), addressee.realName() );
-  tmpl.setValue( TAG( "CLIENT_ORGANISATION" ), addressee.organization() );
-  tmpl.setValue( TAG( "CLIENT_URL" ), addressee.url().prettyUrl() );
-  tmpl.setValue( TAG( "CLIENT_EMAIL" ), addressee.preferredEmail() );
-  tmpl.setValue( TAG( "CLIENT_PHONE" ), addressee.phoneNumber( KABC::PhoneNumber::Work ).number() );
-  tmpl.setValue( TAG( "CLIENT_FAX" ), addressee.phoneNumber( KABC::PhoneNumber::Fax ).number() );
-  tmpl.setValue( TAG( "CLIENT_CELL" ), addressee.phoneNumber( KABC::PhoneNumber::Cell ).number() );
-  KABC::Address clientAddress;
-  clientAddress = addressee.address( KABC::Address::Pref );
-  tmpl.setValue( TAG( "CLIENT_POSTBOX" ),
-                 clientAddress.postOfficeBox() );
-  tmpl.setValue( TAG( "CLIENT_EXTENDED" ),
-                 clientAddress.extended() );
-  tmpl.setValue( TAG( "CLIENT_STREET" ),
-                 clientAddress.street() );
-  tmpl.setValue( TAG( "CLIENT_LOCALITY" ),
-                 clientAddress.locality() );
-  tmpl.setValue( TAG( "CLIENT_REGION" ),
-                 clientAddress.region() );
-  tmpl.setValue( TAG( "CLIENT_POSTCODE" ),
-                 clientAddress.postalCode() );
-  tmpl.setValue( TAG( "CLIENT_COUNTRY" ),
-                 clientAddress.country() );
-  tmpl.setValue( TAG( "CLIENT_REGION" ),
-                 clientAddress.region() );
-  tmpl.setValue( TAG( "CLIENT_LABEL" ),
-                 clientAddress.label() );
+  contactToTemplate( &tmpl, "CLIENT", mCustomerContact );
+  contactToTemplate( &tmpl, "MY", myContact );
 
   tmpl.setValue( TAG( "DOCID" ),   mArchDoc->ident() );
   tmpl.setValue( TAG( "PROJECTLABEL" ),   mArchDoc->projectLabel() );
@@ -270,14 +249,13 @@ void ReportGenerator::addressReceived( KJob* job )
   tmpl.setValue( TAG( "BRUTTOSUM" ), mArchDoc->bruttoSum().toString( mArchDoc->locale() ) );
   tmpl.setValue( TAG( "NETTOSUM" ),  mArchDoc->nettoSum().toString( mArchDoc->locale() ) );
 
-
-  h.setNum( mArchDoc->tax(), 'f', 1 );
+  h = mArchDoc->locale()->formatNumber( mArchDoc->tax() );
   kDebug() << "Tax in archive document: " << h;
   if ( mArchDoc->reducedTaxSum().toLong() > 0 ) {
     tmpl.createDictionary( DICT( "SECTION_REDUCED_TAX" ) );
     tmpl.setValue( "SECTION_REDUCED_TAX", TAG( "REDUCED_TAX_SUM" ),
       mArchDoc->reducedTaxSum().toString( mArchDoc->locale() ) );
-    h.setNum( mArchDoc->reducedTax(), 'f', 1 );
+    h = mArchDoc->locale()->formatNumber( mArchDoc->reducedTax() );
     tmpl.setValue( "SECTION_REDUCED_TAX", TAG( "REDUCED_TAX" ), h );
     tmpl.setValue( "SECTION_REDUCED_TAX", TAG( "REDUCED_TAX_LABEL" ), i18n( "reduced VAT" ) );
   }
@@ -285,68 +263,78 @@ void ReportGenerator::addressReceived( KJob* job )
     tmpl.createDictionary( DICT( "SECTION_FULL_TAX" ) );
     tmpl.setValue( "SECTION_FULL_TAX", TAG( "FULL_TAX_SUM" ),
       mArchDoc->fullTaxSum().toString( mArchDoc->locale() ) );
-    h.setNum( mArchDoc->tax(), 'f', 1 );
+    h = mArchDoc->locale()->formatNumber( mArchDoc->tax() );
     tmpl.setValue( "SECTION_FULL_TAX", TAG( "FULL_TAX" ), h );
     tmpl.setValue( "SECTION_FULL_TAX", TAG( "FULL_TAX_LABEL" ), i18n( "VAT" ) );
   }
 
-  h.setNum( mArchDoc->tax(), 'f', 1 );
+  h = mArchDoc->locale()->formatNumber( mArchDoc->tax() );
   tmpl.setValue( TAG( "VAT" ), h );
 
   tmpl.setValue( TAG( "VATSUM" ), mArchDoc->taxSum().toString( mArchDoc->locale() ) );
 
   // My own contact data
 
-  tmpl.setValue( TAG( "MY_NAME" ), myAddress.realName() );
-  tmpl.setValue( TAG( "MY_ORGANISATION" ), myAddress.organization() );
-  tmpl.setValue( TAG( "MY_URL" ), myAddress.url().prettyUrl() );
-  tmpl.setValue( TAG( "MY_EMAIL" ), myAddress.preferredEmail() );
-  tmpl.setValue( TAG( "MY_PHONE" ), myAddress.phoneNumber( KABC::PhoneNumber::Work ).number() );
-  tmpl.setValue( TAG( "MY_FAX" ), myAddress.phoneNumber( KABC::PhoneNumber::Fax ).number() );
-  tmpl.setValue( TAG( "MY_CELL" ), myAddress.phoneNumber( KABC::PhoneNumber::Cell ).number() );
-
-  KABC::Address address;
-  address = myAddress.address( KABC::Address::Pref );
-  if( address.isEmpty() )
-    address = myAddress.address(KABC::Address::Work );
-  if( address.isEmpty() )
-    address = myAddress.address(KABC::Address::Home );
-  if( address.isEmpty() )
-    address = myAddress.address(KABC::Address::Postal );
-
-  tmpl.setValue( TAG( "MY_POSTBOX" ),
-              address.postOfficeBox() );
-
-  tmpl.setValue( TAG( "MY_EXTENDED" ),
-                 address.extended() );
-  tmpl.setValue( TAG( "MY_STREET" ),
-                 address.street() );
-  tmpl.setValue( TAG( "MY_LOCALITY" ),
-                 address.locality() );
-  tmpl.setValue( TAG( "MY_REGION" ),
-                 address.region() );
-  tmpl.setValue( TAG( "MY_POSTCODE" ),
-                 address.postalCode() );
-  tmpl.setValue( TAG( "MY_COUNTRY" ),
-                 address.country() );
-  tmpl.setValue( TAG( "MY_REGION" ),
-                 address.region() );
-  tmpl.setValue( TAG( "MY_LABEL" ),
-                 address.label() );
-
   QString output = tmpl.expand();
 
   emit templateGenerated( output );
 
 }
 
+#define ADDRESS_TAG( PREFIX, TAG ) QString("%1_%2").arg( PREFIX ).arg( TAG )
+
+void ReportGenerator::contactToTemplate( TextTemplate *tmpl, const QString& prefix, const KABC::Addressee& contact )
+{
+  if( contact.isEmpty() ) return;
+
+  tmpl->setValue( ADDRESS_TAG( prefix, "NAME" ),  contact.realName() );
+  QString co = contact.organization();
+  if( co.isEmpty() ) {
+    co = contact.realName();
+  }
+  tmpl->setValue( ADDRESS_TAG( prefix, "ORGANISATION" ), co );
+  tmpl->setValue( ADDRESS_TAG( prefix, "URL" ),   contact.url().prettyUrl() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "EMAIL" ), contact.preferredEmail() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "PHONE" ), contact.phoneNumber( KABC::PhoneNumber::Work ).number() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "FAX" ),   contact.phoneNumber( KABC::PhoneNumber::Fax ).number() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "CELL" ),  contact.phoneNumber( KABC::PhoneNumber::Cell ).number() );
+
+  KABC::Address address;
+  address = contact.address( KABC::Address::Pref );
+  if( address.isEmpty() )
+    address = contact.address(KABC::Address::Work );
+  if( address.isEmpty() )
+    address = contact.address(KABC::Address::Home );
+  if( address.isEmpty() )
+    address = contact.address(KABC::Address::Postal );
+
+  tmpl->setValue( ADDRESS_TAG( prefix, "POSTBOX" ),
+              address.postOfficeBox() );
+
+  tmpl->setValue( ADDRESS_TAG( prefix, "EXTENDED" ),
+                 address.extended() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "STREET" ),
+                 address.street() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "LOCALITY" ),
+                 address.locality() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "REGION" ),
+                 address.region() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "POSTCODE" ),
+                 address.postalCode() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "COUNTRY" ),
+                 address.country() );
+  tmpl->setValue( ADDRESS_TAG( prefix, "REGION" ),
+                 address.region() );
+  tmpl->setValue( ADDRESS_TAG( prefix,"LABEL" ),
+                 address.label() );
+
+
+}
+
+
 QString ReportGenerator::escapeTrml2pdfXML( const QString& str ) const
 {
-  QString re( Qt::escape( str ) );
-
-  // FIXME: Workaround for broken trml2pdf which needs double escaped
-  //        & characters to work properly.
-  return re.replace( QChar( '&' ), "&amp;" );
+  return( Qt::escape( str ) );
 }
 
 
@@ -365,47 +353,60 @@ QString ReportGenerator::rmlString( const QString& str, const QString& paraStyle
   return rml;
 }
 
-QString ReportGenerator::findTrml2Pdf( )
+QStringList ReportGenerator::findTrml2Pdf( )
 {
   const QString rmlbinDefault = QString::fromLatin1( "trml2pdf" ); // FIXME: how to get the default value?
   QString rmlbin = KraftSettings::self()->trml2PdfBinary();
   kDebug() << "### Start searching rml2pdf bin: " << rmlbin;
 
-  mHaveMerge = false;
+  QStringList retList;
+  mHavePdfMerge = false;
 
   if ( rmlbinDefault == rmlbin  ) {
-    QStringList pathes;
-    KStandardDirs stdDirs;
-    pathes = stdDirs.systemPaths();
-
-    for ( QStringList::Iterator it = pathes.begin(); it != pathes.end(); ++it ) {
-      QString cPath = ( *it ) + "/trml2pdf_kraft.sh";
-      kDebug() << "### Checking cPath: " << cPath;
-      if ( QFile::exists( cPath ) ) {
-        rmlbin = cPath;
-        kDebug() << "Found trml2pdf_kraft.sh in filesystem: " << rmlbin;
-        mHaveMerge = true;
-        break;
+    QString ermlpy = KStandardDirs::locate( "data", "kraft/tools/erml2pdf.py" );
+    kDebug() << "Ermlpy: " << ermlpy;
+    if( ! ermlpy.isEmpty() ) {
+      // need the python interpreter
+      // First check for python2 in python3 times. 
+      QString python = KStandardDirs::findExe("python2");
+      if( python.isEmpty() ) {
+	python = KStandardDirs::findExe("python");
+      }
+      if( python.isEmpty() ) {
+        kError() << "ERR: Unable to find python, thats a problem";
+      } else {
+        kDebug() << "Using python: " << python;
+        retList << python;
+        retList << ermlpy;
+        mHavePdfMerge = true;
+      }
+    } else {
+      // tool erml2pdf.py not found. Try trml2pdf_kraft.sh for legacy reasons
+      QString trml2pdf = KStandardDirs::findExe("trml2pdf_kraft.sh");
+      if( trml2pdf.isEmpty() ) {
+        kDebug() << "Could not find trml2pdf_kraft.sh";
+      } else {
+        kDebug() << "Found trml2pdf: " << trml2pdf;
+        retList << trml2pdf;
+        mHavePdfMerge = true;
       }
     }
 
-    if ( ! mHaveMerge ) {
-      for ( QStringList::Iterator it = pathes.begin(); it != pathes.end(); ++it ) {
-        QString cPath = ( *it ) + "/trml2pdf";
-        if ( QFile::exists( cPath ) ) {
-          rmlbin = cPath;
-          kDebug() << "Found trml2pdf in filesystem: " << rmlbin;
-          break;
-        }
+    if ( ! mHavePdfMerge ) {
+      QString trml2pdf = KStandardDirs::findExe( "trml2pdf");
+      if( trml2pdf.isEmpty() ) {
+        kDebug() << "trml2pdf is also empty, we can not convert rml. Debug!";
+      } else {
+        kDebug() << "trml2pdf found here: " << trml2pdf;
+        retList << trml2pdf;
       }
     }
   }
-  if ( rmlbinDefault == rmlbin  ) {
+  if ( retList.isEmpty() ) {
     kDebug() << "We have not found the script!";
-    rmlbin = QString();
   }
 
-  return rmlbin;
+  return retList;
 }
 
 
@@ -416,9 +417,11 @@ void ReportGenerator::runTrml2Pdf( const QString& rmlFile, const QString& docID,
   QStringList prg;
 
   mErrors = QString();
-  QString rmlbin = findTrml2Pdf();
+  // findTrml2Pdf returns a list of command line parts for the converter, such as
+  // /usr/bin/pyhton /usr/local/share/erml2pdf.py
+  QStringList rmlbin = findTrml2Pdf();
 
-  if ( ! QFile::exists( rmlbin ) ) {
+  if ( ! rmlbin.size() ) {
 
     KMessageBox::error( 0, i18n("The utility to create PDF from the rml file could not be found, "
                                 "but is required to create documents."
@@ -429,9 +432,8 @@ void ReportGenerator::runTrml2Pdf( const QString& rmlFile, const QString& docID,
 
   QApplication::setOverrideCursor( QCursor( Qt::BusyCursor ) );
 
-  if ( mHaveMerge && mMergeIdent != "0" &&
+  if ( mHavePdfMerge && mMergeIdent != "0" &&
        ( mWatermarkFile.isEmpty() || !QFile::exists( mWatermarkFile ) ) ) {
-
     KMessageBox::error( 0, i18n("The Watermark file to merge with the document could not be found. "
                                 "Merge is going to be disabled." ),
                         i18n( "Watermark Error" ) );
@@ -440,20 +442,39 @@ void ReportGenerator::runTrml2Pdf( const QString& rmlFile, const QString& docID,
 
   QString outputDir = ArchiveMan::self()->pdfBaseDir();
   QString filename = ArchiveMan::self()->archiveFileName( docID, archId, "pdf" );
-  mOutFile = QString( "%1/%2" ).arg( outputDir ).arg( filename );
+  mFile.setFileName( QString( "%1/%2").arg( outputDir).arg( filename ) );
 
-  kDebug() << "Writing output to " << mOutFile;
+  kDebug() << "Writing output to " << mFile.fileName();
+
+  // check if we have etrml2pdf
+  bool haveErml = false;
+  if( rmlbin.size() > 1 ) {
+    QString ermlbin = rmlbin[1];
+    if( ermlbin.endsWith( "erml2pdf.py") ) {
+      haveErml = true;
+    }
+  }
 
   prg << rmlbin;
-  if ( mHaveMerge ) {
-    prg << mMergeIdent;
-  }
-  prg << rmlFile;
-  if ( mHaveMerge && mMergeIdent != "0" ) {
-    prg << mWatermarkFile;
+
+  if( !haveErml ) {
+    if ( mHavePdfMerge ) {
+        prg << mMergeIdent;
+    }
+    prg << rmlFile;
+    if ( mHavePdfMerge && mMergeIdent != "0" ) {
+      prg << mWatermarkFile;
+    }
+  } else {
+    kDebug() << "Erml2pdf available!";
+    if ( mHavePdfMerge && mMergeIdent != "0" ) {
+      prg << "-m" << mMergeIdent;
+      prg << "-w" << mWatermarkFile;
+    }
+    prg << rmlFile;
   }
 
-  mFile.setFileName( mOutFile );
+  mFile.setFileName( mFile.fileName() );
   mOutputSize = 0;
   if ( mFile.open( QIODevice::WriteOnly ) ) {
     mProcess.setProgram( prg );
@@ -490,15 +511,15 @@ void ReportGenerator::trml2pdfFinished( int exitStatus)
   kDebug() << "PDF Creation Process finished with status " << exitStatus;
   kDebug() << "Wrote bytes to the output file: " << mOutputSize;
   if ( exitStatus == 0 ) {
-    emit pdfAvailable( mOutFile );
-    mOutFile = QString();
+    emit pdfAvailable( mFile.fileName() );
+    mFile.setFileName( QString() );
   } else {
     if ( mErrors.isEmpty() ) mErrors = i18n( "Unknown problem." );
     // KMessageBox::detailedError (QWidget *parent, const QString &text, const QString &details, const QString &caption=QString::null, int options=Notify)
     KMessageBox::detailedError ( 0,
-                                 i18n( "Could not generate the pdf file. The trml2pdf script failed." ),
+                                 i18n( "Could not generate the pdf file. The pdf creation script failed." ),
                                  mErrors,
-                                 i18n( "rml2pdf Error" ) );
+                                 i18n( "PDF Generation Error" ) );
     mErrors = QString();
   }
   QApplication::restoreOverrideCursor();
